#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""This module contains Enums for the Sentinel-2 bands and some derived 
products.
"""

from enum import Enum, unique

@unique
class Properties(Enum):
    TCI = 'TCI'
    B01 = 'B01'
    B02 = 'B02'
    B03 = 'B03'
    B04 = 'B04'
    B05 = 'B05'
    B06 = 'B06'
    B07 = 'B07'
    B08 = 'B08'
    B8A = 'B8A'
    B09 = 'B09'
    B11 = 'B11'
    B12 = 'B12'
    AOT = 'AOT'
    WVP = 'WVP'
    SCL = 'SCL'
    
    def describe(self): 
        properties = {
            'TCI': {
                'resolution': 10,
                'title': 'True color image',
                'center wavelength': None,
            },
            'B01': {
                'resolution': 60,
                'title': 'Band 1 (coastal)',
                'center wavelength': 0.4439,
            },
            'B02': {
                'resolution': 10,
                'title': 'Band 2 (blue)',
                'center wavelength': 0.4966,
            },
            'B03': {
                'resolution': 10,
                'title': 'Band 3 (green)',
                'center wavelength': 0.56,
            },
            'B04': {
                'resolution': 10,
                'title': 'Band 4 (red)',
                'center wavelength': 0.6645,
            },
            'B05': {
                'resolution': 20,
                'title': 'Band 5',
                'center wavelength': 0.7039,
            },
            'B06': {
                'resolution': 20,
                'title': 'Band 6',
                'center wavelength': 0.7402,
            },
            'B07': {
                'resolution': 20,
                'title': 'Band 7',
                'center wavelength': 0.7825,
            },
            'B08': {
                'resolution': 10,
                'title': 'Band 8 (nir)',
                'center wavelength': 0.8351,
            },
            'B8A': {
                'resolution': 20,
                'title': 'Band 8A',
                'center wavelength': 0.8648,
            },
            'B09': {
                'resolution': 60,
                'title': 'Band 9',
                'center wavelength': 0.945,
            },
            'B11': {
                'resolution': 20,
                'title': 'Band 11 (swir16)',
                'center wavelength': 1.6137,
            },
            'B12': {
                'resolution': 20,
                'title': 'Band 12 (swir22)',
                'center wavelength': 2.22024,
            },
            'AOT': {
                'resolution': 60,
                'title': 'Aerosol Optical Thickness (AOT)',
                'center wavelength': None,
            },
            'WVP': {
                'resolution': 10,
                'title': 'Water Vapour (WVP)',
                'center wavelength': None,
            },
            'SCL': {
                'resolution': 20,
                'title': 'Scene Classification Map (SCL)',
                'center wavelength': None,
            },
        }
        return properties[self.name]
        


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
'''
This module contains some utils used mainly by the library itself.
'''

from typing import Union
import datetime as dt

def _iter_dates(
    start_date: Union[dt.date, dt.datetime], 
    end_date: Union[dt.date, dt.datetime]
):
    '''Helper function to create a (year, month) iterator from start_date to
    end_date.'''
    # REF: https://stackoverflow.com/a/5734564
    ym_start= 12*start_date.year + start_date.month - 1
    ym_end= 12*end_date.year + end_date.month - 1
    for ym in range( ym_start, ym_end + 1):
        y, m = divmod(ym, 12)
        yield y, m+1



# -*- coding: utf-8 -*-

import json
import datetime as dt
from typing import Union, Iterable
from pathlib import Path


import mgrs
import s3fs

from .utils import _iter_dates
from .products import Properties


def download_S2(
    lon: float, 
    lat: float, 
    start_date: Union[dt.date, dt.datetime], 
    end_date: Union[dt.date, dt.datetime], 
    what: Union[str, Iterable[str]],
    cloud_cover_le: float = 50,
    folder: Union[str, Path] = Path.home()
):
    '''Download Sentinel 2 COG (Cloud Optimized GeoTiff) images from Amazon S3. 
    
    The dataset on AWS contains all of the scenes in the original Sentinel-2 
    Public Dataset and will grow as that does. L2A data are available from 
    April 2017 over wider Europe region and globally since December  2018. Read 
    more at the url https://registry.opendata.aws/sentinel-2-l2a-cogs/
    
    Parameters
    ----------
    lon: float
        Float value defining the longitude of interest.
    lat: float
        Float value defining the latitude of interest.
    start_date: datetime.date or datetime.datetime
        Date to start looking for images to download.
    end_date: datetime.date or datetime.datetime
        Date to end looking for images to download.
    what: str or array_like
        Here you have to define what you want to download as a string or as an
        array_like of strings. Valid values are:
            'TCI', 'B01', 'B02', 'B03', 'B04', 'B05', 'B06', 'B07', 'B08',
            'B8A', 'B09', 'B11', 'B12', 'AOT', 'WVP', 'SCL'
    cloud_cover_le: float
        FLoat indicating the maximum cloud cover allowed. If the value is 10
        it indicates the allowed cloud cover on the image must be lower or
        equal to 10%. Default value is 50 (%).
    folder: str or Path
        Where to download the data. The folder must exist. Default value is 
        the home directory of the user.
    
    Returns
    -------
    list
        A list with the paths of the downloaded files.
    '''
    if start_date > end_date:
        raise ValueError(
            '`start_date` has to be lower or equal than `end_date`'
        )
    if isinstance(what, str):
        what = [what]
    for w in what:
        if w.upper() not in [item.value for item in Properties]:
            raise ValueError(f'{w} is not a valid product')
    fs = s3fs.S3FileSystem(anon=True, use_ssl=False)
    m = mgrs.MGRS()
    coord = m.toMGRS(lat, lon, MGRSPrecision=0)
    number, a, b = coord[:-3], coord[-3:-2], coord[-2:]
    start_date = dt.date(start_date. year, start_date.month, start_date.day)
    end_date = dt.date(end_date. year, end_date.month, end_date.day)
    contents = []
    for y, m in _iter_dates(start_date, end_date):
        path = f'sentinel-cogs/sentinel-s2-l2a-cogs/{number}/{a}/{b}/{y}/{m}'
        _contents = fs.ls(path)
        for _c in _contents:
            name = _c.split('/')[-1]
            info = _c + '/' + name + '.json'
            with fs.open(info, 'r') as f:
                info = json.load(f)
            date_str = name.split('_')[2]
            cc = info['properties']['eo:cloud_cover']
            date = dt.datetime.strptime(date_str, '%Y%m%d').date()
            if cloud_cover_le >= cc and start_date <= date <= end_date:
                for w in what:
                    path = _c + f'/{w}.tif'
                    contents.append(path)
                    with fs.open(path, 'rb') as f:
                        data = f.read()
                    path = Path(folder) / f'{name}_{w}.tif'
                    with open(path, 'wb') as f:
                        f.write(data)
    return sorted(contents)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Note: To use the 'upload' functionality of this file, you must:
#   $ pipenv install twine --dev

import io
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = 's2froms3'
DESCRIPTION = (
    'Get Sentinel-2 (Cloud Optimized Geotiffs) COG files from AWS S3.'
)
URL = 'https://github.com/kikocorreoso/s2froms3'
EMAIL = 'kikocorreoso@example.com'
AUTHOR = 'Kiko Correoso'
REQUIRES_PYTHON = '>=3.7,<3.9'
VERSION = '0.1.0'

# What packages are required for this module to be executed?
REQUIRED = ['mgrs', 's3fs']

# What packages are optional?
EXTRAS = {
    'tests': ['pytest'],
}

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION


# Where the magic happens:
setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    package_dir={"": "src"},
    packages=find_packages("src"),
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license='MIT',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3', 
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython'
    ],
)
